/* $Id: base_window.cpp 29 2009-01-02 13:36:32Z mawerick $ */
/* AOCrayon - Chat color management tool for Anarchy Online
	Copyright (C) 2005 - 2009  Mawerick

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; version 2 of the License.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "aocrayon.h"
#include "settings.h"
#include "aocolors.h"
#include "main_window.h"
#include "advmode_page.h"
#include "basicmode_page.h"
#include "settings_page.h"
#include "about_page.h"
#include "license_page.h"

#include <QtGui>

/*
 *  Constructs a MainWindow as a child of 'parent'
 */
MainWindow::MainWindow( QWidget* parent )
		: QMainWindow( parent )
{
	setWindowFlags( Qt::WindowMinimizeButtonHint | Qt::WindowSystemMenuHint );
	setWindowTitle( PROGRAM " v" VERSION );
	setWindowIcon( QPixmap( ":/Icon" ) );

	setFixedSize( 380, 450 );

	QWidget *centralWidget = new QWidget( this );
	setCentralWidget( centralWidget );

	BaseDlgLayout = new QVBoxLayout( centralWidget );
	BaseDlgLayout->setSpacing( 3 );
	BaseDlgLayout->setMargin( 4 );

	TabWidget = new QTabWidget( centralWidget );

	/*
	 * Widget stack for keeping basic/advanced mode color editing
	 */
	ColorsTab = new QStackedWidget;
	TabWidget->addTab( ColorsTab, tr( "Chat Colors" ) );

	/*
	 * Chat color list (basic mode)
	 */
	BasicModeTab = new BasicModePage( ColorsTab );
	ColorsTab->addWidget( BasicModeTab );

	/*
	 * Chat color list (advanced mode)
	 */
	AdvModeTab = new AdvancedModePage( ColorsTab );
	ColorsTab->addWidget( AdvModeTab );

	/*
	 * Settings
	 */
	SettingsTab = new SettingsPage;
	connect( SettingsTab, SIGNAL( updateBGColor() ), SLOT( updateBGColor() ) );
	connect( SettingsTab, SIGNAL( reloadColors() ), SLOT( reloadColors() ) );
	connect( SettingsTab, SIGNAL( modeChanged(bool) ), SLOT( modeChanged(bool) ) );
	connect( SettingsTab, SIGNAL( styleChanged(const QString&) ), SLOT( styleChanged(const QString&) ) );
	TabWidget->addTab( SettingsTab, tr( "Settings" ) );

	/*
	 * About text
	 */
	AboutTab = new AboutPage;
	TabWidget->addTab( AboutTab, tr( "About..." ) );

	/*
	 * License text
	 */
	LicenseTab = new LicensePage;
	TabWidget->addTab( LicenseTab, tr( "License" ) );

	BaseDlgLayout->addWidget( TabWidget );

	/*
	 * Save and close buttons
	 */
	SysLayout = new QHBoxLayout;
	SysLayout->setSpacing( 3 );
	SysLayout->setMargin( 0 );

	SaveBtn = new QPushButton( tr( "&Save" ), centralWidget );
	SaveBtn->setDefault( true );
	connect( SaveBtn, SIGNAL( clicked() ), SLOT( saveColors() ) );
	SysLayout->addWidget( SaveBtn );

	SysLayout->addStretch( 1 );

	CloseBtn = new QPushButton( tr( "&Close" ), centralWidget );
	connect( CloseBtn, SIGNAL( clicked() ), SLOT( close() ) );
	SysLayout->addWidget( CloseBtn );

	BaseDlgLayout->addLayout( SysLayout );

	/* confirm the AO path. if the user clicks cancel, close the program */
	if( SettingsTab->confirmAOPath() == false )
		exit( -1 );

	/* if we failed to load prefs, we assume this is the first time the app is used
	 * and force a backup of what we believe is the original file */
	if( Prefs.get( "FirstRun" ).toBool() == true )
	{
		SettingsTab->backupOriginalColors();
		Prefs.set( "FirstRun", false );
	}

	updateBGColor();
	reloadColors();
	modeChanged( Prefs.get( "AdvancedMode" ).toBool() );
}

void MainWindow::closeEvent( QCloseEvent *event )
{
	Prefs.set( "Position", pos() );
	event->accept();
}

void MainWindow::saveColors()
{
	Colors.save();
}

void MainWindow::reloadColors()
{
	qDebug( "Loading colors." );
	Colors.load();
	AdvModeTab->updateColors();
	BasicModeTab->updateColors();
}

void MainWindow::updateBGColor()
{
	qDebug( "Updating Background color." );
	QColor color( Prefs.get( "BackgroundColor" ).toString() );
	AdvModeTab->setBGColor( color );
	BasicModeTab->setBGColor( color );
	SettingsTab->setBGColor( color );
}

void MainWindow::modeChanged( bool mode )
{
	ColorsTab->setCurrentIndex( mode );
	AdvModeTab->updateColors();
	BasicModeTab->updateColors();
}

void MainWindow::styleChanged( const QString &style )
{
	qDebug( "styleChanged(): %s == %s?? %s", qPrintable( style ),
			qPrintable( qApp->style()->objectName() ),
			style.compare( qApp->style()->objectName(),
						   Qt::CaseInsensitive ) == 0 ? "true" : "false" );

	if( style.compare( qApp->style()->objectName(), Qt::CaseInsensitive ) == 0 )
		return;

	Prefs.set( "GUIStyle", style );

	qDebug( "Hiding GUI" );
	hide();
	qDebug( "Settings style" );
	QApplication::setStyle( style );
	qDebug( "Showing GUI" );
	show();
	update();
	qDebug( "Style change done" );
}
